@@ -96,7 +96,9 @@
 #' When a new subset of degrees of freedom is generated by either a backward or a forward step, the response vector in \code{object}
 #' is fitted to the new model. The fit is carried out by \code{\link{lmvar}}. The arguments used in the call to
 #' \code{lmvar} (other than \code{X_mu}
-#' and \code{X_sigma}) are the same as used to create \code{object}.
+#' and \code{X_sigma}) are the same as used to create \code{object}, except that the control options
+#' \code{mu_full_rank} and \code{sigma_full_rank} are both set to TRUE. Setting them to TRUE can be done safely
+#' because the model matrices \code{object$X_mu} and \code{object$X_sigma} are guaranteed to be full-rank.
 #' }
 #'
 #' @seealso
@@ -215,10 +217,10 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
           # invert Hessian
           H = tryCatch( solve(H),
                         error = function(e){
-                          return("error")
+                          return(NA)
                         })
 
-          if (class(H) == "character"){
+          if (inherits( H, "logical")){
             return(NA)
           }
           else{
@@ -259,10 +261,10 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
           # invert Hessian
           H = tryCatch( solve(H),
                         error = function(e){
-                          return("error")
+                          return(NA)
                         })
 
-          if (class(H) == "character"){
+          if (inherits( H, "logical")){
             return(NA)
           }
           else{
@@ -309,7 +311,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
     if(intercept){
       cols = colnames(X)
       X = X[,-1]
-      if (class(X) == "numeric"){
+      if (inherits( X, "numeric")){
         X = as.matrix(X)
         colnames(X) = cols[2]
       }
@@ -351,7 +353,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
   }
 
   # check inputs
-  if (!("lmvar_no_fit" %in% class(object))){
+  if (!inherits( object, "lmvar_no_fit")){
     stop("object must be of class 'lmvar' or 'lmvar_no_fit'")
   }
   if (missing(fun)){
@@ -368,6 +370,9 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
   if (!("plot" %in% names(control))){
     control$plot = FALSE
   }
+  object_control = object$control
+  object_control$mu_full_rank = TRUE
+  object_control$sigma_full_rank = TRUE
 
   # set whether inserts and removals are done
   if (fw){
@@ -400,11 +405,12 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
 
     iterobject = lmvar( object$y, X_mu, X_sigma, intercept_mu = object$intercept_mu,
                         intercept_sigma = object$intercept_sigma, sigma_min = object$sigma_min,
-                        slvr_options = slvr_options(), control = object$control)
+                        slvr_options = slvr_options(), control = object_control)
   }
   else {
-    if ("lmvar" %in% class(object)){
+    if (inherits( object, "lmvar")){
       iterobject = object
+      iterobject$control = object_control
     }
     else {
 
@@ -413,7 +419,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
 
       iterobject = lmvar( object$y, X_mu, X_sigma, intercept_mu = object$intercept_mu,
                           intercept_sigma = object$intercept_sigma, sigma_min = object$sigma_min,
-                          slvr_options = object$slvr_options, control = object$control)
+                          slvr_options = object$slvr_options, control = object_control)
     }
   }
 
@@ -504,7 +510,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         cols = c( cols, col_mu)
         cols = colnames(iterobject$X_mu) %in% cols
         X_mu = iterobject$X_mu[, cols]
-        if (class(X_mu) == "numeric"){
+        if (inherits( X_mu, "numeric")){
           X_mu = as.matrix(X_mu)
           colnames(X_mu) = colnames(iterobject$X_mu)[cols]
         }
@@ -515,7 +521,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         colnames_adapted = beta_sigma_names( colnames(iterobject$X_mu), colnames(iterobject$X_sigma))
         cols = colnames_adapted %in% cols
         X_sigma = iterobject$X_sigma[, cols]
-        if (class(X_sigma) == "numeric"){
+        if (inherits( X_sigma, "numeric")){
           X_sigma = as.matrix(X_sigma)
           colnames(X_sigma) = colnames(iterobject$X_sigma)[cols]
         }
@@ -527,7 +533,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         # fit
         fit = lmvar( object$y, X_mu, X_sigma, intercept_mu = object$intercept_mu,
                      intercept_sigma = object$intercept_sigma, sigma_min = object$sigma_min,
-                     slvr_options = slvr_options(), control = object$control)
+                     slvr_options = slvr_options(), control = object_control)
         fun_iter = fun(fit)
 
         # set iteration counter
@@ -593,7 +599,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         col = names(effect_sorted[1:n_insert])
         cols = colnames(object$X_mu) %in% c( colnames(iterobject$X_mu), col)
         X_mu = object$X_mu[, cols]
-        if (class(X_mu) == "numeric"){
+        if (inherits( X_mu, "numeric")){
           X_mu = as.matrix(X_mu)
           colnames(X_mu) = colnames(object$X_mu)[cols]
         }
@@ -602,7 +608,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         col = names(names_sigma[names_sigma %in% col])
         cols = colnames(object$X_sigma) %in% c( colnames(iterobject$X_sigma), col)
         X_sigma = object$X_sigma[, cols]
-        if (class(X_sigma) == "numeric"){
+        if (inherits( X_sigma, "numeric")){
           X_sigma = as.matrix(X_sigma)
           colnames(X_sigma) = colnames(object$X_sigma)[cols]
         }
@@ -614,7 +620,7 @@ fwbw.lmvar_no_fit <- function( object, fun, fw = FALSE, counter = TRUE, df_perce
         # fit
         fit = lmvar( object$y, X_mu, X_sigma, intercept_mu = object$intercept_mu,
                      intercept_sigma = object$intercept_sigma, sigma_min = object$sigma_min,
-                     slvr_options = slvr_options(), control = object$control)
+                     slvr_options = slvr_options(), control = object_control)
         fun_iter = fun(fit)
 
         # set iteration counter
